const _ = require('lodash');
const fs = require('fs-extra');
const oco = require('opencolor')
const strftime = require('strftime');

const config = require('./config');
const utils = require('./utils');
const source = require('./document/source')


var myPaletteDocs = {}
var currentDoc = null
var myOwnWriteFilePath = null


//
// CRUD

function set(palettePath, newDoc) {
  myPaletteDocs[palettePath] = newDoc
}

function get(palettePath) {
  return myPaletteDocs[palettePath]
}

function remove(palettePath) {
  delete myPaletteDocs[palettePath]
}

function reset() {
  myPaletteDocs = {}
}

function isEmpty() {
  return myPaletteDocs.length === 0
}

function forEach(callback) {
  for (var filePath in myPaletteDocs) {
    callback(myPaletteDocs[filePath])
  }
}

//
// Create new doc

function createDiskDocument(fileContent) {
 var doc = {
   newTitle: null,
   codemirrorDoc: source.createDocument(fileContent),
   inMemoryOnly: false,
   isDirty: false,
   isSourcePreview: false,
   isValid: true,
   assistant: false,
   transformPanel: {
     data: {}
   },
   previewScrollTop: 0,
   toolsConfig: {},
   currentTool: null
 }

 return doc
}

function createInMemoryDocument() {
  return {
    path: '__in_memory__',
    title: 'Untitled',
    newTitle: null,
    codemirrorDoc: source.createDocument(''),
    inMemoryOnly: true,
    isDirty: false,
    isSourcePreview: false,
    isValid: true,
    assistant: true,
    transformPanel: {
      data: {}
    },
    previewScrollTop: 0,
    toolsConfig: {},
    currentTool: null
  }
}

function createInMemoryDocumentForImport(importFile) {
  var doc = createInMemoryDocument();

  doc.assistant = false;
  doc.title = importFile.split('/').pop().split('.').shift();
  doc.newTitle = doc.title
  doc.currentTool = 'import';
  doc.toolsConfig['import'] = {
    path: importFile
  }

  return doc;
}

//
// populate

function traverseTree(subtree, path, callback) {
  subtree.forEach(function(entry) {
    if (entry.type === 'Palette') {
      traverseTree(entry, path.concat([entry.name]), callback);
    } else if (entry.type === 'Reference') {
      var resolvedEntry = entry.resolved();
      if (resolvedEntry) {
        callback(path, resolvedEntry);
      }
    } else {
      callback(path, entry);
    }
  });
}

function getColorValueForMetakey(tree, key) {
  var metadata = tree.metadata.get(key);
  if (!metadata) {
    return null;
  }
  if (metadata.type == 'ColorValue') {
    return metadata.value;
  }
  if (metadata.type == 'Reference') {
    var resolvedEntry = metadata.resolved();
    if(!resolvedEntry) {
      return null;
    }
    return resolvedEntry.hexcolor();
  }
  return null;
}

function getColorCount(ocoTree) {
  if (!ocoTree) {
    return "Parse Error"
  }

  let count = 0
  traverseTree(ocoTree, [], (path, entry) => {
    if (entry && entry.type === 'Color') {
      count++;
    }
  });

  if (count === 1) {
    return `${count} Color`;
  } else {
    return `${count} Colors`;
  }

}

function getIdentityColors(ocoTree) {
  if (!ocoTree) {
    return {
      top: '#F00',
      side: '#F33',
      bottom: '#F55'
    }
  }

  const identityColors = {
    top: getColorValueForMetakey(ocoTree, 'oct/identityColorTop'),
    side: getColorValueForMetakey(ocoTree, 'oct/identityColorSide'),
    bottom: getColorValueForMetakey(ocoTree, 'oct/identityColorBottom')
  };

  if (identityColors.top && identityColors.side && identityColors.bottom) {
    return identityColors;
  }

  const firstThree = ['#666', '#333', '#000'];
  let i = 0;
  traverseTree(ocoTree, [], (path, entry) => {
    if (i < 3 && entry.type === 'Color' && entry.hexcolor()) {
      firstThree[i] = entry.hexcolor();
      i++;
    }
  });

  Object.keys(identityColors).forEach((k, index) => {
    if(!identityColors[k]) {
      identityColors[k] = firstThree[index];
    }
  });

  return identityColors;
}

function initDoc(filePath, name) {
  const fileInfo = fs.lstatSync(filePath)
  const fileContent = fs.readFileSync(filePath, 'utf-8');

  const doc = createDiskDocument(fileContent)

  doc.title = name.replace(".oco", "")
  doc.path = filePath

  if (fileInfo.isSymbolicLink()) {
    const linkTargetInfo = fs.statSync(filePath)
    doc.lastModified = strftime('%d %b %Y %H:%M', linkTargetInfo.mtime)
    doc.createdAt = fileInfo.birthtime
    doc.isSymlink = true
  } else {
    doc.lastModified = strftime('%d %b %Y %H:%M', fileInfo.mtime)
    doc.createdAt = fileInfo.birthtime
    doc.isSymlink = true
  }

  parseForDoc(doc, fileContent)

  doc.colorCount = getColorCount(doc.tree)
  doc.identityColors = getIdentityColors(doc.tree)

  return doc
}

function init() {
  var docs = fs.readdirSync(config.settings.get('libfolder')).map(function(name) {
    if (config.isValidFile(name)) {
      const filePath = config.settings.get('libfolder') + name

      return initDoc(filePath, name)
    }
  });

  docs = _.compact(docs)
  docs = _.sortBy(docs, function(doc) { return doc.createdAt }).reverse()

  docs.forEach(function(doc) {
    myPaletteDocs[doc.path] = doc
  })
}

function reorderDocs() {
  var docs = _.values(myPaletteDocs)

  docs = _.sortBy(docs, function(doc) { return doc.createdAt }).reverse()

  myPaletteDocs = {}

  docs.forEach(function(doc) {
    myPaletteDocs[doc.path] = doc
  })
}

function syncDocs(path) {
  const existingDoc = get(path)
  const fileWasDeleted = !utils.pathIsReadable(path)

  if (fileWasDeleted) {
    // delete
    remove(path)
  } else if (existingDoc) {
    // update
    if (myOwnWriteFilePath === path) {
      // inside write
      myOwnWriteFilePath = null
    } else {
      // outside write
      discardChangesToDoc(existingDoc)
    }
  } else {
    // new file
    const name = utils.getNameFromPath(path)
    const newDoc = initDoc(path, name)
    set(path, newDoc)
  }

  reorderDocs()
}

function markAsMyOwnWrite(path) {
  myOwnWriteFilePath = path
}

//
// Current

function current() {
  return currentDoc
}

function setCurrent(newDoc) {
  currentDoc = newDoc
}

function parseForDoc(doc, source) {
  try {
    doc.tree = oco.parse(source);
    doc.isValid = true;
  } catch(err) {
    doc.tree = null;
    doc.isValid = false;
    doc.parseError = err;
  }
}

function parseCurrentDoc() {
  var source = currentDoc.codemirrorDoc.getValue();
  parseForDoc(currentDoc, source)
}

function updateCurrentDocDirtyState() {
  currentDoc.isDirty = currentDoc.codemirrorDoc && currentDoc.codemirrorDoc.isDirty() || currentDoc.newTitle && currentDoc.newTitle != currentDoc.title
}

function discardChangesToDoc(doc) {
  remove(doc.path)

  const newDoc = initDoc(doc.path, doc.title)
  set(newDoc.path, newDoc)
}

function discardChangesToCurrentDoc() {
  remove(currentDoc.path)

  const doc = initDoc(currentDoc.path, currentDoc.title)
  set(doc.path, doc)
  setCurrent(doc)
}


exports.get = get
exports.set = set
exports.remove = remove
exports.reset = reset
exports.isEmpty = isEmpty
exports.forEach = forEach

exports.init = init
exports.syncDocs = syncDocs
exports.markAsMyOwnWrite = markAsMyOwnWrite

exports.current = current
exports.setCurrent = setCurrent
exports.parseCurrentDoc = parseCurrentDoc
exports.discardChangesToCurrentDoc = discardChangesToCurrentDoc
exports.updateCurrentDocDirtyState = updateCurrentDocDirtyState

exports.createInMemoryDocument = createInMemoryDocument
exports.createInMemoryDocumentForImport = createInMemoryDocumentForImport



/*
currentDoc = {
  palette: {
    path
    title
  }
  inMemoryOnly,
  isDirty,
  isSourcePreview,
  isValid, // valid oco
  codemirrorDoc,
  tree,
  parseError,
  sourceBeforePreview,
  toolsConfig: {
    transform: {
      transformFunction,
      $ui // hack
    }
    export,
    share
  },
  selectedTool,
  previews: {
    picker: {
      scrollTop
    }
    export: {

    }
  }
}
*/

/**

## notes on document and ui states

document.codemirrorDoc
  - the document truth is maintained by codemirrorDoc
    as long as currentDoc.isSourcePreview == false
    when isSourcePreview == true the codemirrorDoc 'follows' the
    tree
document.inMemoryOnly
  - exists in memory only
  -> user cannot leave the document without dicarding or saving
document.isDirty
  - unsaved changes in source
  -> user may switch to other documents
document.isValid
  - source is valid oco format
documentTools
  - either: null, import, transform, export or share
  content of documentTools is persistet when switching tools
  documentTools == null
    -> the documentTools are not visible
  documentTools == import
    editor is editable
    preview is a export preview
  documentTools == transform
    document.isSourcePreview is possible
    -> transform function selector and options are visible
  documentTools == export
    editor is editable
    preview is a export preview
document.isSourcePreview
  - the code-editor displays a preview version of the document
  -> user may not edit in editor
  -> user has to stop preview, apply changes or close documentTool before he can save
*/
