const fs = require('fs-extra')
const utils = require('../utils')
const templates = require('../templates')
const readFormOptions = require('./tool-utils').readFormOptions
const switchPromisable = require('./tool-utils').switchPromisable
const ocoConverter = require('opencolor-converter')
const ocoTransformer = require('opencolor-transformer')
const remote = require('electron').remote
const dialog = remote.dialog
const oco = require('opencolor')

var onCancel
var $panel = null
var $exportOptions = null
var exportOptions = {}
var exportTarget = 'scss'
var tree = null
var exportPreviewListener = function () {}

const validExportTargets = {
  'scss': 'SCSS',
  'less': 'LESS',
  'css': 'CSS',
  'json': 'JSON',
  'swift': 'Swift',
  'oco': 'OCO'
}

function createStyleExporter (syntax) {
  return styleExporter = function (tree, options) {
    if (options.allAsVars) {
      options.allAsVars = true
      options.flatten = true
      options.mapProperties = false
    }

    return Promise.resolve(tree)
      .then(
        switchPromisable(
          options.flatten,
          ocoTransformer.flatten.configure({
            glue: '-'
          })
      )).then(
        switchPromisable(
          true,
          ocoTransformer.compoundWords.configure({
            transform: 'dasherize'
          })
      )).then((x) => {
        return Promise.resolve(x)
      }).then(
        ocoConverter[syntax].exporter.configure(options)
      )
  }
}

function createSwiftExporter () {
  return swiftExporter = function (tree, options) {
    options.allAsVars = true
    options.flatten = true

    return Promise.resolve(tree)
      .then(
        switchPromisable(
          options.flatten,
          ocoTransformer.flatten.configure({
            glue: ' '
          })
      )).then(
        switchPromisable(
          true,
          ocoTransformer.compoundWords.configure({
            transform: 'camelize'
          })
      )).then((x) => {
        return Promise.resolve(x)
      }).then(
        ocoConverter.swift.exporter.configure(options)
      )
  }
}

function createOcoExporter () {
  return ocoExporter = function (tree, options) {
    var ocoString = oco.render(tree)
    return Promise.resolve(ocoString)
  }
}

var validExporter = {
  'css': createStyleExporter('css'),
  'less': createStyleExporter('less'),
  'scss': createStyleExporter('scss'),
  'json': ocoConverter.jsonExtended.exporter,
  'swift': createSwiftExporter(),
  'oco': createOcoExporter()
}

function switchExporter () {
  exportTarget = $panel.find('[data-export-target]').val()
  if (Object.keys(validExportTargets).indexOf(exportTarget) === -1) {
    return
  }
  $panel.find('[data-export-target-title]').text(validExportTargets[exportTarget])
  const template = templates.loadTemplate('tools/Export-' + exportTarget + '.hbs')
  const functionOptionsFormContent = '<div>' + template({}) + '</div>'
  $exportOptions.html(functionOptionsFormContent)

  $exportOptions.find('input, select').unbind('keyup.exportFunction change.exportFunction')
  $exportOptions.find('input, select').bind('keyup.exportFunction change.exportFunction', function (e) {
    e.preventDefault()
    exportOptions = readFormOptions($exportOptions)
    performExport()
  })
  exportOptions = readFormOptions($exportOptions)
  performExport()
}

function showError (message) {
  $panel.find('[data-export-apply]').prop('disabled', true)
  $panel.find('[data-export-error]').text(message).show()
}

function hideError () {
  $panel.find('[data-export-apply]').prop('disabled', false)
  $panel.find('[data-export-error]').text('').hide()
}

function performExport (callback) {
  if (!tree) {
    showError('Not able to export invalid palettes')
    return
  }
  validExporter[exportTarget](tree, exportOptions).then((source) => {
    if (callback) {
      callback(source)
    } else {
      exportPreviewListener(source)
      hideError()
    }
  }).catch((err) => {
    showError(err)
  })
}

function update (_config) {
  config = _config
}
function updateTree (_tree) {
  tree = _tree
  performExport()
}

function saveExport () {
  dialog.showSaveDialog(remote.BrowserWindow.getFocusedWindow(),
    {
      title: `Export as ${exportTarget}`,
      filters: [{name: exportTarget, extensions: [exportTarget]}],
      buttonLabel: 'Export'
    }, (path) => {
      if (!path) {
        return
      }
      performExport((source) => {
        fs.writeFile(path, source, 'utf8', (err) => {
          if (err) throw err
          $panel.hide()
          onCancel()
        })
      })
    })
}

function get (_onCancel, _eventBus) {
  onCancel = _onCancel

  $panel = $(templates.exportTool({validExportTargets: validExportTargets}))

  $exportOptions = $panel.find('[data-export-content]')
  switchExporter()

  $panel.find('[data-export-target]').on('change', function (e) {
    switchExporter()
  })
  $panel.find('[data-export-cancel]').on('click', function (e) {
    e.preventDefault()
    $panel.hide()
    onCancel()
  })
  $panel.find('[data-export-apply]').on('click', function (e) {
    e.preventDefault()
    saveExport()
  })
  return $panel
}

function close () {

}

function getConfig () {
  return {}
}

module.exports = {
  get: get,
  update: update,
  updateTree: updateTree,
  close: close,
  getConfig: getConfig
}
