const templates = require('../templates')
const ocoConverter = require('opencolor-converter')
const ocoTransformer = require('opencolor-transformer')
const utils = require('../utils')
const readFormOptions = require('./tool-utils').readFormOptions
const switchPromisable = require('./tool-utils').switchPromisable
const path = require('path')
const Vibrant = require('node-vibrant')

var $panel = null
var source = ''
var fileType = ''
var $importOptions = null
var importOptions = {}
var validImporter = {
  'css': createStyleImporter('css'),
  'less': createStyleImporter('less'),
  'scss': createStyleImporter('scss'),
  'json': {
    importer: jsonImporter,
    sourceReader: readStringSource
  },
  'sketchpalette': {
    importer: sketchpaletteImporter,
    sourceReader: readStringSource
  },
  'aco': {
    importer: acoImporter,
    sourceReader: readBufferedSource
  },
  'png': {
    importer: imageImporter,
    sourceReader: readBufferedSource
  },
  'jpg': {
    importer: imageImporter,
    sourceReader: readBufferedSource
  },
  'jpeg': {
    importer: imageImporter,
    sourceReader: readBufferedSource
  }
}

function readStringSource (filePath) {
  return utils.readFile(filePath, 'utf-8')
}

function readBufferedSource (filePath) {
  return utils.readFile(filePath)
}

function getFilePath (filePath) {
  return filePath
}

function createStyleImporter (syntax) {
  return {
    importer: function (source, options) {
      return ocoConverter[syntax].importer(source, options)
        .then(
          switchPromisable(
            options.beautifyNames,
            ocoTransformer.searchAndReplace.configure({
              search: '-color',
              replace: ''
            })
        )).then(
          switchPromisable(
            options.beautifyNames,
            ocoTransformer.compoundWords.configure({
              transform: 'humanize'
            })
        )).then(
          switchPromisable(
            options.extractRepeating,
            ocoTransformer.abstractRepeating.configure({
              autoname: true
            })
        ))
    },
    sourceReader: readStringSource
  }
}

function jsonImporter (source, options) {
  return ocoConverter.jsonExtended.importer(source, options)
  .then(
    switchPromisable(
      true,
      ocoTransformer.flatten.configure({
        glue: 1,
        maxDepth: 2
      })
  ))
}

function sketchpaletteImporter (source, options) {
  return ocoConverter.json.importer(source, options)
  .then(
    switchPromisable(
      options.autoname,
      ocoTransformer.autoname.configure({})
    ))
}

function acoImporter (buffer, options) {
  return ocoConverter.aco.importer(buffer, options)
  .then(
    switchPromisable(
      options.extractRepeating,
      ocoTransformer.abstractRepeating.configure({
        autoname: true
      })
    ))
}

function imageImporter (source, options) {
  return ocoConverter.image.importer(source, options)
  .then(
    switchPromisable(
      options.autoname,
      ocoTransformer.autoname.configure({})
    ))
}

function switchImporter () {
  const template = templates.loadTemplate('tools/Import-' + fileType + '.hbs')
  const functionOptionsFormContent = '<div>' + template({}) + '</div>'
  $importOptions.html(functionOptionsFormContent)

  $importOptions.find('input, select').unbind('keyup.transformFunction change.transformFunction')
  $importOptions.find('input, select').bind('keyup.transformFunction change.transformFunction', function (e) {
    e.preventDefault()
    importOptions = readFormOptions($importOptions)
    performImport()
  })
  importOptions = readFormOptions($importOptions)
  performImport()
}

function update (config) {
  if (!config) {
    return
  }
  var filePath = config.path || null
  if (filePath) {
    eventBus.sourcePreviewStateChanged(true)
    var newFileType = path.extname(filePath).replace('.', '').toLowerCase()
    if (Object.keys(validImporter).indexOf(newFileType) === -1) {
      throw 'Importer: Invalid file type ' + newFileType
    }
    fileType = newFileType
    source = validImporter[fileType].sourceReader(filePath)
    switchImporter()
    $('[data-import-file]').text(filePath.split('/').pop())
  } else {
    source = ''
    performImport()
  }
}

function showError (message) {
  $panel.find('[data-import-apply]').prop('disabled', true)
  $panel.find('[data-import-error]').text(message).show()
}

function hideError () {
  $panel.find('[data-import-apply]').prop('disabled', false)
  $panel.find('[data-import-error]').text('').hide()
}

function performImport (apply) {
  if (!source) {
    return
  }
  validImporter[fileType].importer(source, importOptions).then((tree) => {
    eventBus.treeChanged(tree, apply)
    hideError()
  }).catch((err) => {
    showError(err)
  })
}

var eventBus

function get (onCancel, _eventBus) {
  eventBus = _eventBus
  eventBus.sourcePreviewStateChanged(true)

  $panel = $(templates.importTool({}))
  $importOptions = $panel.find('[data-import-content]')

  $panel.find('[data-import-cancel]').on('click', function (e) {
    e.preventDefault()
    eventBus.sourcePreviewStateChanged(false)
    $panel.hide()
    onCancel()
    eventBus.documentKilled()
  })

  $panel.find('[data-import-apply]').on('click', function (e) {
    e.preventDefault()
    eventBus.sourcePreviewStateChanged(false)
    performImport(true)
    $panel.hide()
    onCancel()
  })

  return $panel
}

function close () {
}

function getConfig () {
  return {}
}

module.exports = {
  get: get,
  update: update,
  close: close,
  getConfig: getConfig,
  hideToolbar: true
}
