const transformTool = require('./transform-tool');
const exportTool = require('./export-tool');
const shareTool = require('./share-tool');
const importTool = require('./import-tool');


var currentToolName = null;
var currentTool = null;
var config = {};
var tree = null;

const validTools = {
  'import': importTool,
  'transform': transformTool,
  'export': exportTool,
  'share': shareTool
}

const eventBus = {
  treeChanged: function() {},
  sourcePreviewStateChanged: function() {},
  toolChanged: function() {},
  documentKilled: function() {},
}

function setAnimation(animated) {
  if(animated) {
    $('[data-document-panel-footer]').addClass('is-animated');
  } else {
    $('[data-document-panel-footer]').removeClass('is-animated');
  }
}

function close(toolName, animated) {
  setAnimation(animated);
  $('[data-document-panel-footer]').addClass("is-hidden");
  $('[data-document-tools-toggle]').removeClass('is-active');
  if (currentTool) {
    currentTool.close();
  }
  showToolbar();
  eventBus.toolChanged(toolName);
  currentToolName = null;
  currentTool = null;
}

function switchTool(toolName, animated) {
  $('[data-document-tools-toggle]').removeClass('is-active');

  if(toolName == currentToolName) {
    return;
  }
  if(!toolName && currentToolName) {
    // schließbefehl
    close(currentToolName, animated);
    return;
  }

  if(!(toolName in validTools)) {
    throw new Error(`Request unsupported tool '${toolName}'`);
    return;
  }
  if (currentTool) {
    currentTool.close();
  }
  currentToolName = toolName;
  currentTool = validTools[toolName];

  if(currentTool.hideToolbar) {
    hideToolbar();
  } else {
    showToolbar();
  }
  setAnimation(animated);

  $('[data-document-tools-toggle=' + toolName +']').addClass('is-active');
  $('[data-document-panel-footer]').removeClass("is-hidden");

  function onToolCancel() {
    close(toolName, true)
  }
  var $elm = currentTool.get(onToolCancel, eventBus);

  currentTool.update(config[currentToolName] || {});

  if(currentTool.updateTree) {
    currentTool.updateTree(tree);
  }

  $('[data-document-panel-footer]').html('').append($elm);
  eventBus.toolChanged(currentToolName);
}

function selectTool(selectedToolName, config, tree) {
  switchTool(selectedToolName, false);
  update(config);
  updateTree(tree, true);
}

function init(treeChangedListener, sourcePreviewStateChangedListener, toolChangedListener, documentKilledListener) {
  eventBus.treeChanged = treeChangedListener;
  eventBus.sourcePreviewStateChanged = sourcePreviewStateChangedListener;
  eventBus.toolChanged = toolChangedListener;
  eventBus.documentKilled = documentKilledListener;

  $('[data-document-tools-toggle]').click(function() {
    const $trigger = $(this);
    $('[data-document-tools-toggle]').removeClass('is-active');

    var toolName = $trigger.data('document-tools-toggle');
    if(toolName == currentToolName) {
      close(toolName, true);
    } else {
      switchTool(toolName, true);
    }
  });
}

function update(_config) {
  config = _config;
  if(!currentTool) {
    return;
  }
  currentTool.update(config[currentToolName] || {})
}

function updateTree(_tree, initialize) {
  tree = _tree;
  if(!currentTool || !currentTool.updateTree) {
    return;
  }
  currentTool.updateTree(tree, initialize)
}

function getCurrentTool() {
  return currentToolName;
}

function getCurrentConfigForAllTools() {
  const allConfigs = {};
  Object.keys(validTools).forEach(function(toolName) {
    allConfigs[toolName] = validTools[toolName].getConfig();
  })
  return allConfigs;
}

function hideToolbar() {
  $('[data-document-toolbar]').hide();
}

function showToolbar() {
  $('[data-document-toolbar]').show();
}

module.exports = {
  init: init,
  update: update,
  getCurrentConfigForAllTools: getCurrentConfigForAllTools,
  getCurrentTool: getCurrentTool,
  selectTool: selectTool,
  updateTree: updateTree,
  hideToolbar: hideToolbar,
  showToolbar: showToolbar
}
