const fs = require('fs-extra');
const oco = require('opencolor');
const ipcRenderer = require('electron').ipcRenderer;
const remote = require('electron').remote;
const clipboard = require('electron').clipboard;
const dialog = remote.dialog;
const Menu = remote.Menu
const MenuItem = remote.MenuItem


const leftPane = require('./leftpane');
const config = require('./config');
const utils = require('./utils');
const docsManager = require('./docs-manager');
const source = require('./document/source')
const pickerPreview = require('./document/picker-preview')
const tools = require('./document/tools')
const assistant = require('./document/assistant')
const header = require('./document/header')
const titleModal = require('./document/title-modal')

var myCodeMirror;

const noop = () => {};


function parseCurrentDoc() {
  docsManager.parseCurrentDoc()

  tools.updateTree(docsManager.current().tree);
  updateDirtyState();
  updateDocumentActionsUI();
}

function parseAndPreview(delayError) {
  parseCurrentDoc();
  pickerPreview.update(docsManager.current(), delayError);
}

function onSourceChange(codemirrorDoc) {
  parseAndPreview(true);
}

function updateDirtyState() {
  if (!docsManager.current()) {
    return;
  }

  docsManager.updateCurrentDocDirtyState()
  leftPane.updateDocState(docsManager.current())
}

function updateDocumentActionsUI() {
  if(!docsManager.current()) {
    return;
  }
  if (!docsManager.current().isSourcePreview && !docsManager.current().assistant && docsManager.current().isDirty) {
    // allow save
    $('.Document [data-is-dirty]').removeClass("is-hidden");
    $('.Document [data-is-valid]').addClass("is-hidden");
    $('button[data-save]').removeClass('Button--disabled').prop('disabled', false);
    ipcRenderer.send('disableSaveMenuItem', false);
    $('button[data-discard]').removeClass('Button--disabled').prop('disabled', false);
  } else {
    // disallow save
    $('.Document [data-is-dirty]').addClass("is-hidden");
    $('button[data-save]').addClass('Button--disabled').prop('disabled', true);
    ipcRenderer.send('disableSaveMenuItem', true);
    $('button[data-discard]').addClass('Button--disabled').prop('disabled', true);
  }

  if (docsManager.current().isValid) {
    $('.Document [data-is-invalid]').addClass("is-hidden");
  } else {
    $('.Document [data-is-invalid]').removeClass("is-hidden");
  }
  if (docsManager.current().isSourcePreview) {
    $('.Document-header').addClass("is-preview");
    $('.Document [data-is-preview]').removeClass("is-hidden");
  } else {
    $('.Document-header').removeClass("is-preview");
    $('.Document [data-is-preview]').addClass("is-hidden");
  }

  if (docsManager.current().inMemoryOnly) {
    $('.Document-header').addClass("is-inMemoryOnly");
    $('.Document [data-is-inmemoryonly]').removeClass("is-hidden");
  } else {
    $('.Document-header').removeClass("is-inMemoryOnly");
    $('.Document [data-is-inmemoryonly]').addClass("is-hidden");
  }

}

function updateHeaderUI() {
  if(!docsManager.current()) {
    return;
  }
  header.updateUI(docsManager.current());
}

function editTitle() {
  header.startEditTitle();
}

function cancelTitle() {
  header.stopEditTitle();
}

function onSubmitTitle(title) {
  docsManager.current().newTitle = title;
  updateDirtyState();
  updateHeaderUI();
  updateDocumentActionsUI();
}

function onCancelTitle() {
  updateDirtyState();
  updateHeaderUI();
  updateDocumentActionsUI();
}

function getPathForNewName() {
  if(!docsManager.current().newTitle) {
    return false;
  }
  var newName = docsManager.current().newTitle.replace('.oco', '');
  const orgName = newName;
  newName = newName.replace(/\//g, '-');
  if (newName.trim() == '') {
    alert(`Error – Invalid title '${orgName}'`);
    editTitle();
    return false;
  }

  const targetPath = config.settings.get('libfolder') + newName + '.oco';
  if(utils.pathIsReadable(targetPath)) {
    alert(`Error – A Palette with the title '${newName}' already exists.`);
    editTitle();
    return false;
  }
  return targetPath;
}

function savePalette(callback) {
  callback = callback || noop;
  const path = docsManager.current().path;

  if(path == '__in_memory__') {
    function inMemoryDocSaved(success) {
      if(success) {
        destroyInMemoryDocument();
      }
      callback(success)
    }
    function saveWithNewName() {
      var newPath = getPathForNewName();
      if(!newPath) {
        callback(false);
        return;
      }
      savePaletteAtPath(newPath, inMemoryDocSaved);
    }
    if(!docsManager.current().newTitle) {
      source.lock();
      titleModal.openDialog('Untitled', function(name) {
        if(name) {
          docsManager.current().newTitle = name;
          saveWithNewName();
        } else {
          callback(false);
          return;
        }
      });
    } else {
      // if titled
      saveWithNewName();
    }
  } else {
    if(docsManager.current().newTitle && docsManager.current().newTitle != docsManager.current().title) {
      var newPath = getPathForNewName();
      if(!newPath) {
        callback(false);
        return;
      }
      fs.renameSync(docsManager.current().path, newPath);
      savePaletteAtPath(newPath, callback);
    } else {
      savePaletteAtPath(path, callback);
    }
  }

}

function savePaletteAtPath(path, callback) {
  callback = callback || noop;
  const data = docsManager.current().codemirrorDoc.getValue();

  leftPane.setCurrentFile(path)
  docsManager.markAsMyOwnWrite(path)

  fs.writeFile(path, data, 'utf8', (err) => {
    if (err) throw err;
    docsManager.current().codemirrorDoc.markAsClean();
    docsManager.current().newTitle = null;
    updateDirtyState();
    updateDocumentActionsUI();
    parseAndPreview();

    callback(true);
  });
}

function discardPalette() {
  if(docsManager.current().inMemoryOnly) {

    const dialogOptions = {
      type: 'question',
      buttons: ['Throw away', 'Cancel'],
      message: `Throw away unsaved palette`,
      detail: `Are you sure you want to throw away all changes in ${docsManager.current().title}?`,
    }

    dialog.showMessageBox(remote.BrowserWindow.getFocusedWindow(), dialogOptions, function(buttonIndex) {
      if (buttonIndex === 0) {
        destroyInMemoryDocument();
        leftPane.selectFirstPalette();
      }
    })

  } else {
    docsManager.discardChangesToCurrentDoc()
    selectPalette(docsManager.current())
  }
}

function destroyInMemoryDocument() {
  if(!docsManager.current().inMemoryOnly) {
    return;
  }
  assistant.close();
  docsManager.remove('__in_memory__')
  docsManager.setCurrent(null)
}

function canLeaveCurrentDoc() {
  if (!docsManager.current()) {
    return true;
  }

  if (!docsManager.current().inMemoryOnly) {
    // no problem file is on disk
    return true;
  }

  if (docsManager.current().assistant) {
    // no problem, leaving the assistant
    return true;
  }

  return false;
}

function askToSave(callback) {
  const dialogOptions = {
    type: 'question',
    buttons: ['Save', 'Discard', 'Cancel'],
    message: `Save changes to ${docsManager.current().title}`,
    detail: 'Do you want to save the new document?',
  }

  dialog.showMessageBox(remote.BrowserWindow.getFocusedWindow(), dialogOptions, function(buttonIndex) {
    if (buttonIndex === 0) {
      // save
      savePalette(callback);
    } else if (buttonIndex === 1) {
      destroyInMemoryDocument();
      callback(true);
    } else {
      callback(false);
    }
  })
}

function selectPalette(doc, callback) {
  if (!canLeaveCurrentDoc()) {
    askToSave(function(mayLeave) {
      if (mayLeave) {
        selectPalette(doc, callback);
      } else {
        callback(false);
      }
    });

    return;
  }

  // handle state of currently opend document, before selecting new palette
  if (docsManager.current()) {
    if (docsManager.current().inMemoryOnly) {
      // all saftey checks passed
      destroyInMemoryDocument();
    } else {
      docsManager.current().previewScrollTop = $('[data-preview]').scrollTop()
      docsManager.current().toolsConfig = tools.getCurrentConfigForAllTools();
      docsManager.current().currentTool = tools.getCurrentTool();
    }
    cancelTitle();
  }

  if (!doc) {
    docsManager.setCurrent(docsManager.createInMemoryDocument())
  } else if (doc.importFile) {
    docsManager.setCurrent(docsManager.createInMemoryDocumentForImport(doc.importFile))
  } else {
    docsManager.setCurrent(doc)
  }

  source.getCodeMirror().swapDoc(docsManager.current().codemirrorDoc);
  parseCurrentDoc()
  pickerPreview.update(docsManager.current());

  $('[data-preview]').scrollTop(docsManager.current().previewScrollTop);
  updateDocumentActionsUI();
  updateHeaderUI();

  if (docsManager.current().assistant) {
    tools.hideToolbar();
    source.lock();
    assistant.show();
  } else {
    source.unlock();
  }

  if (docsManager.current().currentTool == 'import') {
    leftPane.deselectPalette();
  }

  tools.selectTool(docsManager.current().currentTool, docsManager.current().toolsConfig, docsManager.current().tree);

  if (callback) {
    callback(true);
  }

  return true;
}

function onAssistantClose(ocoString) {
  docsManager.current().assistant = false;
  tools.showToolbar();
  updateDocumentActionsUI();
  if(ocoString) {
    source.update(ocoString);
  }
  if(ocoString === false) {
    destroyInMemoryDocument();
    leftPane.selectFirstPalette();
  }
  source.unlock();
}

function onToolUpdatesTree(tree, apply) {
  if (!apply && docsManager.current().isSourcePreview) {
    docsManager.current().tree = tree;
    source.update(oco.render(tree), true);
    pickerPreview.update(docsManager.current());
  } else if (apply) {
    source.endPreviewMode();
    source.update(oco.render(tree));
    docsManager.current().sourceBeforePreview = null
    docsManager.current().isSourcePreview = false;
    parseAndPreview();
  }
}

function onToolUpdatesSourcePreviewState(previewState) {
  docsManager.current().isSourcePreview = previewState;
  if(docsManager.current().isSourcePreview) {
    var currentSource = source.startPreviewMode();
    if (!docsManager.current().sourceBeforePreview) {
      docsManager.current().sourceBeforePreview = currentSource;
    }
    pickerPreview.update(docsManager.current());
  } else {
    source.endPreviewMode(docsManager.current().sourceBeforePreview);
    docsManager.current().sourceBeforePreview = null;
    parseAndPreview();
  }
  updateDocumentActionsUI();
}

function onToolChange() {

}

function reset() {
  docsManager.reset()
}

function onToolKillsDocument() {
  destroyInMemoryDocument();
  leftPane.selectFirstPalette();
}

var fileListItemWithContextMenu = null;

function initContextMenu() {
  const menu = new Menu()

  menu.append(new MenuItem({
    label: 'Copy Hex',
    click: function() {
        var colorHex = fileListItemWithContextMenu.data('color-hexvalue');
        clipboard.writeText(colorHex);
    }
  }))


  window.addEventListener('contextmenu', (e) => {
    e.preventDefault()

    fileListItemWithContextMenu = null

    var fileListElm = $(document.elementFromPoint(e.x, e.y)).closest('[data-preview] .OctColor');
    if (fileListElm.length) {
      fileListItemWithContextMenu = fileListElm;
      menu.popup(remote.getCurrentWindow())
    }
  }, false)
}

function init() {
  source.init();
  source.setChangeListener(onSourceChange);
  tools.init(onToolUpdatesTree, onToolUpdatesSourcePreviewState, onToolChange, onToolKillsDocument);
  assistant.init(onAssistantClose);
  header.init(onSubmitTitle, onCancelTitle);

  $('body').on('click', 'button[data-save]', (e) => {
    e.preventDefault();
    savePalette();
  });
  $('body').on('click', 'button[data-discard]', discardPalette);

  ipcRenderer.on('saveFile', () => {
    savePalette();
  });

  initContextMenu()

  // $('body').on('click', '[data-preview] .OctColor', function() {
  //   var colorHex = $(this).data('color-hexvalue');
  //   clipboard.writeText(colorHex);
  // });

}


exports.init = init;
exports.reset = reset;
exports.selectPalette = selectPalette;
exports.editTitle = editTitle;
