const path = require('path')
const ocoConverter = require('opencolor-converter')
const ipcRenderer = require('electron').ipcRenderer
const remote = require('electron').remote
const dialog = remote.dialog

const config = require('./config')
const leftPane = require('./leftpane')
const editor = require('./editor')
const utils = require('./utils')
const templates = require('./templates')

var dragging = false
var selectingFile = false

function allowDrag (e) {
  const file = e.originalEvent.dataTransfer.files[0]

  if (file && config.isValidFileForImport(file.name)) {
    e.originalEvent.dataTransfer.dropEffect = 'copy'
  } else {
    e.originalEvent.dataTransfer.dropEffect = 'none'
  }
  e.preventDefault()
}

function fileDropped (e) {
  e.preventDefault()
  dragging = false
  var file = e.originalEvent.dataTransfer.files[0]
  handleFile(file.path)
}

function showChooseFileDialog () {
  dialog.showOpenDialog({
    properties: ['openFile'],
    filters: [
      { name: 'Open Color File', extensions: ['oco'] },
      { name: 'Importable', extensions: ['less', 'css', 'scss', 'sketchpalette'] } // TODO Add JSON
    ]
  }, fileSelected)
}

function fileSelected (paths) {
  if (!paths) {
    return
  }

  const filePath = paths[0]
  handleFile(filePath)
}

function fileAlreadyInLibraryFolder (filePath) {
  return filePath.startsWith(config.settings.get('libfolder'))
}

function copyOrLinkFile (filePath) {
  const dialogOptions = {
    type: 'info',
    buttons: ['Copy File', 'Cancel'],
    message: `Add "${path.basename(filePath)}" to Library`,
    detail: `Copying the file will create a duplicate of it in the library folder.\n`
  }

  dialog.showMessageBox(dialogOptions, function (buttonIndex) {
    if (buttonIndex === 0) {
      // Copy File
      utils.copyFile(filePath)
    } else if (buttonIndex === 1) {
      // Link File
      // utils.symlinkFile(filePath)
    } else if (buttonIndex === 2) {
      // Cancel
    }
  })
}

function handleFile (filePath) {
  $('[data-file-drop]').hide()
  closeDropzoneModal()
  closeSelectFileDialog()

  if (path.extname(filePath) === '.oco') {
    if (fileAlreadyInLibraryFolder(filePath)) {
      leftPane.selectPaletteWithPath(filePath)
      return
    }
    copyOrLinkFile(filePath)
  } else {
    editor.selectPalette({importFile: filePath})
  }
}

var dropzoneModalTimer

function openDropzoneModal () {
  var data = {}
  var $modal = $(templates.dropzoneModal(data))

  $('[data-app-modal-container]').append($modal).removeClass('is-hidden')

  clearTimeout(dropzoneModalTimer)
  dropzoneModalTimer = setTimeout(function () {
    $modal.removeClass('is-hidden')
  }, 30)
}

function closeDropzoneModal () {
  $('[data-dropzone]').addClass('is-hidden')

  clearTimeout(dropzoneModalTimer)
  dropzoneModalTimer = setTimeout(function () {
    $('[data-dropzone]').remove()
    $('[data-app-modal-container]').addClass('is-hidden')
  }, 300)
}

function openSelectFileDialog () {
  var data = {}
  var $modal = $(templates.selectFileModal(data))

  $modal.find('[data-cancel]').on('click', function (e) {
    e.preventDefault()
    closeSelectFileDialog()
  })

  $modal.find('[data-submit]').on('click', showChooseFileDialog)

  $('[data-app-modal-container]').append($modal).removeClass('is-hidden')

  setTimeout(function () {
    $modal.removeClass('is-hidden')
  }, 30)
}

function closeSelectFileDialog () {
  selectingFile = false
  $('[data-select-file-overlay]').addClass('is-hidden')

  setTimeout(function () {
    $('[data-select-file-overlay]').remove()
    $('[data-app-modal-container]').addClass('is-hidden')
  }, 300)
}

function init () {
  dragging = false
  var dragOverlay = $('[data-file-drop]')

  window.addEventListener('dragenter', function (e) {
    if (dragging == false) {
      $('[data-file-drop]').show()
      $('body').trigger('fieldropdragenter.oct')
      if (!$('body').data('assistant')) {
        openDropzoneModal()
      }
    }
    dragging = true
  })

  dragOverlay.on('dragover', allowDrag)

  dragOverlay.on('dragleave', function (e) {
    $('[data-file-drop]').hide()
    $('body').trigger('fieldropdragleave.oct')
    closeDropzoneModal()
    dragging = false
  })

  dragOverlay.on('drop', fileDropped)

  ipcRenderer.on('importFileFromMenu', function () {
    showChooseFileDialog()
  })

  ipcRenderer.on('openFile', function (event, filePath) {
    handleFile(filePath)
  })

  $(document).keydown(function (e) {
    // ESCAPE key pressed
    if (e.keyCode == 27) {
      closeSelectFileDialog()
    }
  })
}

exports.init = init
exports.showChooseFileDialog = showChooseFileDialog
