"use strict";


const shell = require('electron').shell;
const ipcRenderer = require('electron').ipcRenderer;
const remote = require('electron').remote;
const dialog = remote.dialog;
const Menu = remote.Menu
const MenuItem = remote.MenuItem

const docsManager = require('./docs-manager');
const editor = require('./editor');
const templates = require('./templates');
const utils = require('./utils');


var currentFile = null;
var fileListItemWithContextMenu = null;


function setCurrentFile(filePath) {
  currentFile = filePath;
}

function render() {
  if (docsManager.isEmpty()) {
    $('#drop-overlay').show();
  } else {
    $('[data-DocumentListItem]').remove();

    const $documentList = $('[data-DocumentList-List]');

    docsManager.forEach(function(doc) {
      $documentList.append(templates.documentListItem(doc));
    })
  }

  if ((currentFile) && $(`[data-palette-path="${currentFile}"]`).length) {
    selectPalette($(`[data-palette-path="${currentFile}"]`));
  } else {
    selectFirstPalette();
  }
}

function selectFirstPalette() {
  selectPalette($('[data-DocumentListItem]:first'))
}

function deselectPalette() {
  $('[data-DocumentListItem].is-selected').removeClass('is-selected');
}

function selectPaletteWithPath(palettePath) {
  const $palette = $(`[data-palette-path="${palettePath}"]`)
  if ($palette.length) {
    selectPalette($palette)
  }
}

function selectPalette(elmPalette, callback) {
  const filePath = elmPalette.data('palette-path');

  editor.selectPalette(docsManager.get(filePath), function(selected) {
    if (!selected) {
      if (callback) {
        callback(false);
      }
      return;
    }

    $('[data-DocumentListItem].is-selected').removeClass('is-selected');
    elmPalette.addClass('is-selected');
    currentFile = filePath;
    if (callback) {
      callback(true);
    }
  });
}

function updateDocState(doc) {
  var $elm = $('[data-palette-path="' + doc.path + '"]');
  if (!$elm.size()) {
    return;
  }

  if (doc.isDirty) {
    $elm.addClass("is-dirty");
  } else {
    $elm.removeClass("is-dirty");
  }
}

function renameFile() {

  if (!fileListItemWithContextMenu) {
    return
  }
  selectPalette($(fileListItemWithContextMenu), (selected) => {
    if(selected) {
      editor.editTitle();
    }
  });
}

function confirmDeleteFile() {
  if (!fileListItemWithContextMenu) {
    return
  }

  const filePath = fileListItemWithContextMenu.data('palette-path');
  const fileName = fileListItemWithContextMenu.data('palette-title');

  const dialogOptions = {
    type: 'question',
    buttons: ['Delete', 'Cancel'],
    message: `Delete "${fileName}"`,
    detail: 'Are you sure you want to delete this file?',
  }

  dialog.showMessageBox(dialogOptions, function(buttonIndex) {
    if (buttonIndex === 0) {
      docsManager.remove(filePath)
      utils.deleteFile(filePath)
    }
  })
}

function showFileInFinder() {
  if (!fileListItemWithContextMenu) {
    return
  }

  const filePath = fileListItemWithContextMenu.data('palette-path');
  shell.showItemInFolder(filePath);
}

function initContextMenu() {
  const menu = new Menu()

  menu.append(new MenuItem({
    label: 'Rename',
    click: renameFile
  }))

  menu.append(new MenuItem({
    label: 'Show in Finder',
    click: showFileInFinder
  }))

  menu.append(new MenuItem({type: 'separator'}))

  menu.append(new MenuItem({
    label: 'Delete',
    click: confirmDeleteFile
  }))

  window.addEventListener('contextmenu', (e) => {
    e.preventDefault()

    fileListItemWithContextMenu = null

    var fileListElm = $(document.elementFromPoint(e.x, e.y)).closest('[data-DocumentListItem]');
    if (fileListElm.length) {
      fileListItemWithContextMenu = fileListElm;
      menu.popup(remote.getCurrentWindow())
    }
  }, false)
}

function filterDocumentList() {
  var searchString = $(".DocumentList-Filter input[data-DocumentList-Filter]").val().toLowerCase();

  if (searchString.length > 0) {
    $("[data-DocumentList-Filter-clear]").show();
  } else {
    $("[data-DocumentList-Filter-clear]").hide();
  }

  $("[data-DocumentListItem]").each(function() {
    var title = $(this).attr("data-palette-title").toLowerCase();

    if (title.indexOf(searchString) > -1) {
      $(this).show();
    } else {
      $(this).hide();
    }
  });
}

function clearFilter() {
  $("input[data-DocumentList-Filter]").val("");
  filterDocumentList();
}

function init() {
  render()

  $('body').on('click', '[data-DocumentListItem]', function() {
    selectPalette($(this));
  });

  $("input[data-DocumentList-Filter]").on('keyup', filterDocumentList);
  $("[data-DocumentList-Filter-clear]").on('click', clearFilter);

  ipcRenderer.on('filterDocumentList', function() {
    $("input[data-DocumentList-Filter]").focus();
  });

  $(document).keydown(function(e) {
    // ESCAPE key pressed
    if (e.keyCode == 27) {
      // Blur search
      $("input[data-DocumentList-Filter]").blur();
      clearFilter();
    }
  });

  initContextMenu();

  initNewFile();
}

function openNewFile() {
  editor.selectPalette(null, function(selected) {
    if(!selected) {
      return;
    }
    currentFile = '__in_memory__';
    $('[data-DocumentListItem].is-selected').removeClass('is-selected');
  });
}

function initNewFile() {
  $('body').on('click', '[data-new-palette]', function(e) {
    e.preventDefault();
    openNewFile();
  });
  ipcRenderer.on('newFile', function() {
    openNewFile();
  });
}


exports.init = init;
exports.setCurrentFile = setCurrentFile;
exports.render = render;
exports.updateDocState = updateDocState;
exports.selectPaletteWithPath = selectPaletteWithPath;
exports.selectFirstPalette = selectFirstPalette;
exports.deselectPalette = deselectPalette;
