const fs = require('fs-extra');
const path = require('path');
const shell = require('electron').shell;
const ipcRenderer = require('electron').ipcRenderer;
const child_process = require('child_process');
const remote = require('electron').remote;
const dialog = remote.dialog;
const installer = require('./installer');
const config = require('./config');
const templates = require('./templates');
const utils = require('./utils');


function isDropboxAvailable() {
  return utils.pathIsWritable(getDropboxFolderPath());
}

function isDocumentsAvailable() {
  return utils.pathIsWritable(config.documentsFolderPath);
}

function getDropboxFolderPath() {
  // https://www.dropbox.com/help/4584
  const dropboxInfoPath = path.join(config.homedir, '.dropbox', 'info.json');
  const dropBoxInfo = utils.readJsonFile(dropboxInfoPath)
  if (!dropBoxInfo) {
    return null
  }

  if ('business' in dropBoxInfo) {
    return path.resolve(dropBoxInfo['business']['path']);
  }

  if ('personal' in dropBoxInfo) {
    return path.resolve(dropBoxInfo['personal']['path']);
  }

  return null;
}

function isSketchInstalled() {
  try {
    child_process.execSync('/System/Library/Frameworks/CoreServices.framework/Versions/A/Frameworks/LaunchServices.framework/Versions/A/Support/lsregister -dump | grep -q com.bohemiancoding.sketch3');
    return true;
  } catch (err) {
    return false;
  }
}

function isSketchRunning() {
  try {
    child_process.execSync('pgrep -x Sketch')
    return true
  } catch (err) {
    return false
  }
}

function setFilePickerPath(path) {
  var readablePath = path;
  if (path) {
    readablePath = path.replace(config.homedir, '~');
  }
  $('.Onboarding .FilePicker input').data('absolute-path', path);
  $('.Onboarding .FilePicker input').val(readablePath);
}

function storeLibFolderPath(path, callback) {
  if (!path) {
    return;
  }

  const libFolderPath = path;

  if (utils.pathIsWritable(libFolderPath)) {
    config.settings.set('libfolder', libFolderPath);
    callback(libFolderPath);
  } else {
    dialog.showErrorBox('Setup Error', "Couldn't access the library folder: " + libFolderPath);
  }
}

function libFolderChosen() {
  const path = $('.Onboarding .FilePicker input').data('absolute-path');

  if (!path) {
    $('.Onboarding button[libFolderChosen]').prop('disabled', true);
    $('.Onboarding button[libFolderChosen]').addClass("Button--disabled");
    dialog.showErrorBox('Setup Error', 'Please select a valid path first');
    return;
  }

  utils.ensurePath(path);

  storeLibFolderPath(path, function() {
    const libFolderPath = config.settings.get('libfolder');
    const existingFiles = fs.readdirSync(libFolderPath);

    if (existingFiles.some(config.isValidFile)) {
      // there are already oco files in the folder
    } else {
      console.log("Installing sample palettes");
      installer.installSamplePalettes(libFolderPath);
    }

    if (isSketchInstalled()) {
      $('#Onboarding-screen-libfolder').addClass("is-hidden");
      $('#Onboarding-screen-plugin').removeClass("is-hidden");
    } else {
      endOnboarding()
    }
  });
}

function installSketchPlugin() {
  installer.installSketchPlugin();

  $('#Onboarding-screen-plugin').addClass("is-hidden");
  $('#Onboarding-screen-tutorial').removeClass("is-hidden");
}

function skipSketchPlugin() {
  endOnboarding();
}

function startSketchTutorial() {
  if (isSketchRunning()) {
    const dialogOptions = {
      type: 'question',
      buttons: ['Restart Sketch', 'Skip Sketch Tutorial'],
      message: `Sketch is running`,
      detail: `Unfortunately there is no way around restarting Sketch before you can go through the tutorial.`,
    }
    dialog.showMessageBox(dialogOptions, function(buttonIndex) {
      if (buttonIndex === 0) {
        restartSketch(launchSketchTutorial, skipSketchTutorial)
      } else {
        skipSketchTutorial()
      }
    })
  } else {
    launchSketchTutorial()
  }
}

function restartSketch(onSuccessCallback, onFailureCallback) {
  try {
    child_process.execSync("osascript -e 'tell application \"Sketch\" to activate' -e 'tell application \"Sketch\" to quit'")
    child_process.execSync("sleep 2")
    onSuccessCallback()
  } catch (err) {
    onFailureCallback()
  }
}

function launchSketchTutorial() {
  const sketchTutorialFile = fs.realpathSync(__dirname + '/../resources/OpenColorTutorial.sketch')
  shell.openItem(sketchTutorialFile)

  endOnboarding()
}

function skipSketchTutorial() {
  endOnboarding();
}

function askAboutTracking() {
  $('#Onboarding-screen-tracking').removeClass("is-hidden");
}

function askAboutTrackingCompleted() {
  $('#Onboarding-screen-tracking').addClass("is-hidden");
  startLibfolder()
}

function startOnboarding() {
  $('body').on('click', '.Onboarding button[chooseLibFolder]', chooseLibFolder);
  $('body').on('click', '.Onboarding button[libFolderChosen]', libFolderChosen);

  $('body').on('click', '.Onboarding button[installSketchPlugin]', installSketchPlugin);
  $('body').on('click', '.Onboarding button[skipSketchPlugin]', skipSketchPlugin);

  $('body').on('click', '.Onboarding button[startSketchTutorial]', startSketchTutorial);
  $('body').on('click', '.Onboarding button[skipSketchTutorial]', skipSketchTutorial);

  $('body').on('click', '.Onboarding button[askAboutTrackingCompleted]', askAboutTrackingCompleted);

  var $onbaording = $(templates.onboarding({}))
  $('[data-app-modal-container]').append($onbaording)
  $('[data-app-modal-container]').removeClass("is-hidden")

  askAboutTracking()
}

function startLibfolder() {
  var libFolderPath = config.settings.get('libfolder');
  if (!libFolderPath) {
    var suggestion = null;

    if (isDropboxAvailable()) {
      suggestion = getDropboxFolderPath() + '/Open Color Library/';
    } else if(isDocumentsAvailable()) {
      suggestion = config.documentsFolderPath + '/Open Color Library/';
    }

    if (suggestion) {
      libFolderPath = suggestion;
      $('.Onboarding button[libFolderChosen]').prop('disabled', false);
      $('.Onboarding button[libFolderChosen]').removeClass("Button--disabled");
    }
  }

  setFilePickerPath(libFolderPath);

  $('#Onboarding-screen-libfolder').removeClass('is-hidden')
}

function endOnboarding() {
  $('[data-onboarding]').remove();
  $('[data-app-modal-container]').addClass("is-hidden");

  installer.ensureEverythingUpToDate();
  successCallback();
}

function chooseLibFolder() {
  openLibFolderDialog(function(paths) {
    if (!paths) {
      return;
    }

    const libFolderPath = paths[0] + '/';
    setFilePickerPath(libFolderPath);

    $('.Onboarding button[libFolderChosen]').prop('disabled', false);
    $('.Onboarding button[libFolderChosen]').removeClass("Button--disabled");
  })
}

function changeLibraryFolder() {
  openLibFolderDialog(function(paths) {
    if (!paths) {
      return;
    }

    const libFolderPath = paths[0] + '/';
    storeLibFolderPath(libFolderPath, function(libFolderPath) {
      remote.getCurrentWindow().reload();
    });
  })
}

function openLibFolderDialog(callback) {
  var defaultPath = config.settings.get('libfolder');

  if (!defaultPath && isDropboxAvailable()) {
    defaultPath = getDropboxFolderPath();
  } else if(!defaultPath && isDocumentsAvailable()) {
    defaultPath = config.documentsFolderPath;
  } else if(!defaultPath) {
    defaultPath = null;
  }

  dialog.showOpenDialog({
    defaultPath: defaultPath,
    properties: ['openDirectory', 'createDirectory'],
  }, callback);
}

var successCallback;

function init(callback) {
  successCallback = callback;

  if (installer.installCompleted()) {
    // normal app start
    installer.ensureEverythingUpToDate();
    successCallback();
  } else {
    // start onboarding
    startOnboarding();
  }

  ipcRenderer.on('changeLibraryFolder', changeLibraryFolder);
}


exports.init = init;
