'use strict';

const fs = require('fs');
const electron = require('electron');
const ipcMain = require('electron').ipcMain;
const _ = require('lodash');
const app = electron.app;
const BrowserWindow = electron.BrowserWindow;
const Menu = electron.Menu;
const windowStateKeeper = require('electron-window-state');

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow;

let applicationMenu;
let droppedFile;


function createApplicationMenu() {
  var appMenu = {
    label: 'Electron',
    submenu: [
      {
        label: 'About Open Color Companion',
        role: 'about'
      },
      {
        type: 'separator'
      },
      {
        label: "Change Library Folder…",
        click: function(item, focusedWindow) {
          mainWindow.webContents.send('changeLibraryFolder');
        }
      },
      {
        type: 'separator'
      },
      {
        label: 'Hide Open Color Companion',
        accelerator: 'Command+H',
        role: 'hide'
      },
      {
        label: 'Hide Others',
        accelerator: 'Command+Alt+H',
        role: 'hideothers'
      },
      {
        label: 'Show All',
        role: 'unhide'
      },
      {
        type: 'separator'
      },
      {
        label: 'Quit Open Color Companion',
        accelerator: 'Command+Q',
        click: function() { app.quit() }
      }
    ]
  }

  var fileMenu = {
    label: "File",
    submenu: [
      {
        label: "New Palette",
        accelerator: "CmdOrCtrl+N",
        click: function(item, focusedWindow) {
          mainWindow.webContents.send('newFile');
        }
      },
      {
        label: "Save",
        enabled: false,
        id: "save",
        accelerator: "CmdOrCtrl+S",
        click: function(item, focusedWindow) {
          mainWindow.webContents.send('saveFile');
        }
      },
      { type: 'separator' },
      {
        label: "Import…",
        accelerator: "CmdOrCtrl+I",
        //enabled: false,
        click: function(item, focusedWindow) {
          mainWindow.webContents.send('importFileFromMenu');
        }
      }
    ]
  }

  var editMenu = {
    label: "Edit",
    submenu: [
      { label: "Undo", accelerator: "CmdOrCtrl+Z", selector: "undo:" },
      { label: "Redo", accelerator: "Shift+CmdOrCtrl+Z", selector: "redo:" },
      { type: "separator" },        { label: "Cut", accelerator: "CmdOrCtrl+X", selector: "cut:" },
      { label: "Copy", accelerator: "CmdOrCtrl+C", selector: "copy:" },
      { label: "Paste", accelerator: "CmdOrCtrl+V", selector: "paste:" },
      { label: "Select All", accelerator: "CmdOrCtrl+A", selector: "selectAll:" },
      { type: "separator" },
      {
        label: "Filter Palettes",
        accelerator: "CmdOrCtrl+F",
        click: function(item, focusedWindow) {
          mainWindow.webContents.send('filterDocumentList');
        }
      },
    ]
  }


  var viewMenu =  {
    label: 'View',
    submenu: [
      // {
      //   label: "Toggle Dark Mode",
      //   click: function(item, focusedWindow) {
      //     mainWindow.webContents.send('darkMode');
      //   }
      // },
      {
        label: 'Toggle Full Screen',
        checked: true,
        accelerator: 'Ctrl+Command+F',
        click: function(item, focusedWindow) {
          if (focusedWindow) focusedWindow.setFullScreen(!focusedWindow.isFullScreen())
        }
      }
    ]
  }

  if (process.env.NODE_ENV === 'development') {
    viewMenu.submenu.unshift(
      {
        label: 'Reload',
        accelerator: 'CmdOrCtrl+R',
        click: function(item, focusedWindow) {
          if (focusedWindow) focusedWindow.reload()
        }
      },
      {
        label: 'Toggle Developer Tools',
        accelerator: 'Alt+Command+I',
        click: function(item, focusedWindow) {
          if (focusedWindow) focusedWindow.toggleDevTools()
        }
      },
      {
        type: 'separator'
      })
  }

  var windowMenu = {
    label: "Window",
    role: "window",
    submenu: [
      { label: "Minimize", accelerator: "CmdOrCtrl+M", role: "minimize" },
      { type: 'separator' },
      { label: "Bring All To Front", role: "front" }
    ]
  }

  var helpMenu = {
    label: "Help",
    role: "help",
    submenu: [
      {
        label: "Open Tutorial.sketch",
        click: function() {
          var sketchTutorialFile = fs.realpathSync(__dirname + '/resources/OpenColorTutorial.sketch');
          electron.shell.openItem(sketchTutorialFile);
        }
      },
        {
          type: 'separator'
        },
        {
          label: "Go to the Support Page",
          click: function() { electron.shell.openExternal('http://opencolor.tools/support.html') }
        },
        {
          label: "Get in Touch",
          click: function() { electron.shell.openExternal('mailto:support@opencolor.tools') }
        }
    ]
  }

  applicationMenu = Menu.buildFromTemplate([appMenu, fileMenu, editMenu, viewMenu, windowMenu, helpMenu])
  Menu.setApplicationMenu(applicationMenu)
}

function createWindow () {
  let mainWindowState = windowStateKeeper({
     defaultWidth: 920,
     defaultHeight: 580
   });


  // Create the browser window.
  mainWindow = new BrowserWindow({
    x: mainWindowState.x,
    y: mainWindowState.y,
    width: mainWindowState.width,
    height: mainWindowState.height,
    minWidth: 760,
    minHeight: 500
  });

  mainWindowState.manage(mainWindow);

  // and load the index.html of the app.
  mainWindow.loadURL('file://' + __dirname + '/index.html');

  mainWindow.webContents.on('did-finish-load', function() {
    if (droppedFile) {
      mainWindow.webContents.send('openFile', droppedFile)
      droppedFile = null
    }
  });

  createApplicationMenu()

  // Open the DevTools.
  // mainWindow.webContents.openDevTools();

  // Emitted when the window is closed.
  mainWindow.on('closed', function() {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null;
  });
}

function flattenMenuItems(menu) {
  let items = []
  for (var index in menu.items) {
    let item = menu.items[index]

    items.push(item)
    if (item.submenu) {
      items = items.concat(flattenMenuItems(item.submenu))
    }
  }

  return items
}

function disableSaveMenuItem(state) {
  if (!applicationMenu) {
    return
  }

  let menu = flattenMenuItems(applicationMenu);

  let saveMenuItem = _.find(menu, item => item.id === 'save')

  if (state == true) {
    saveMenuItem.enabled = false;
  } else {
    saveMenuItem.enabled = true;
  }
}

ipcMain.on('disableSaveMenuItem', function(event, arg) {
  disableSaveMenuItem(arg);
});

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
app.on('ready', createWindow);

app.on('open-file', function(event, path) {
  if (mainWindow) {
    mainWindow.webContents.send('openFile', path);
  } else {
    droppedFile = path;
  }
});

// Quit when all windows are closed.
app.on('window-all-closed', function () {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', function () {
  // On OS X it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (mainWindow === null) {
    createWindow();
  }
});
